@@ -1,9 +1,11 @@
 from ykman.device import list_all_devices, read_info
 from ykman.pcsc import list_devices
+from ykman._cli.util import find_scp11_params
 from yubikit.core import TRANSPORT, Version
 from yubikit.core.otp import OtpConnection
 from yubikit.core.fido import FidoConnection
 from yubikit.core.smartcard import SmartCardConnection
+from yubikit.core.smartcard.scp import ScpKid
 from functools import partial
 from . import condition
 
@@ -21,6 +23,20 @@ def _device(pytestconfig):
             serial = None
         else:
             pytest.skip("No serial specified for device tests")
+
+    version_str = pytestconfig.getoption("use_version")
+    if version_str:
+        version = Version.from_string(version_str)
+
+        # Monkey patch all parsing of Version to use the supplied value
+        # N.B. There are some instances where ideally we would replace the version,
+        # but we don't really care
+        def get_version(cls, data):
+            return version
+
+        Version.from_bytes = classmethod(get_version)
+        Version.from_string = classmethod(get_version)
+
     reader = pytestconfig.getoption("reader")
     if reader:
         readers = list_devices(reader)
@@ -36,9 +52,6 @@ def _device(pytestconfig):
         dev, info = devices[0]
     if info.serial != serial:
         pytest.exit("Device serial does not match: %d != %r" % (serial, info.serial))
-    version = pytestconfig.getoption("use_version")
-    if version:
-        info.version = Version.from_string(version)
 
     return dev, info
 
@@ -100,3 +113,11 @@ def ccid_connection(device, info):
             yield c
     else:
         pytest.skip("CCID connection not available")
+
+
+@pytest.fixture(scope=connection_scope)
+def scp_params(ccid_connection):
+    try:
+        return find_scp11_params(ccid_connection, ScpKid.SCP11b, 0)
+    except ValueError:
+        return None
