@@ -37,6 +37,7 @@ from fido2.ctap2 import (
 )
 from fido2.pcsc import CtapPcscDevice
 from yubikit.management import CAPABILITY
+from yubikit.core import TRANSPORT
 from yubikit.core.fido import FidoConnection
 from yubikit.core.smartcard import SW
 from time import sleep
@@ -98,14 +99,18 @@ def info(ctx):
     """
     Display general status of the FIDO2 application.
     """
-    conn = ctx.obj["conn"]
+    info = ctx.obj["info"]
     ctap2 = ctx.obj.get("ctap2")
-    info: Dict = {}
-    lines: List = [info]
 
-    if is_yk4_fips(ctx.obj["info"]):
-        info["FIPS Approved Mode"] = "Yes" if is_in_fips_mode(conn) else "No"
-    elif ctap2:
+    data: Dict = {}
+    lines: List = [data]
+
+    if CAPABILITY.FIDO2 in info.fips_capable:
+        data["FIPS approved"] = CAPABILITY.FIDO2 in info.fips_approved
+    elif is_yk4_fips(info):
+        data["FIPS approved"] = is_in_fips_mode(ctx.obj["conn"])
+
+    if ctap2:
         client_pin = ClientPin(ctap2)  # N.B. All YubiKeys with CTAP2 support PIN.
         if ctap2.info.options["clientPin"]:
             if ctap2.info.force_pin_change:
@@ -115,42 +120,41 @@ def info(ctx):
                 )
             pin_retries, power_cycle = client_pin.get_pin_retries()
             if pin_retries:
-                info["PIN"] = f"{pin_retries} attempt(s) remaining"
+                data["PIN"] = f"{pin_retries} attempt(s) remaining"
                 if power_cycle:
                     lines.append(
                         "PIN is temporarily blocked. "
                         "Remove and re-insert the YubiKey to unblock."
                     )
             else:
-                info["PIN"] = "blocked"
+                data["PIN"] = "Blocked"
         else:
-            info["PIN"] = "not set"
-        info["Minimum PIN length"] = ctap2.info.min_pin_length
+            data["PIN"] = "Not set"
+        data["Minimum PIN length"] = ctap2.info.min_pin_length
 
         bio_enroll = ctap2.info.options.get("bioEnroll")
         if bio_enroll:
             uv_retries = client_pin.get_uv_retries()
             if uv_retries:
-                info["Fingerprints"] = f"registered, {uv_retries} attempt(s) remaining"
+                data["Fingerprints"] = f"Registered, {uv_retries} attempt(s) remaining"
             else:
-                info["Fingerprints"] = "registered, blocked until PIN is verified"
+                data["Fingerprints"] = "Registered, blocked until PIN is verified"
         elif bio_enroll is False:
-            info["Fingerprints"] = "not registered"
+            data["Fingerprints"] = "Not registered"
 
         always_uv = ctap2.info.options.get("alwaysUv")
         if always_uv is not None:
-            info["Always Require UV"] = "on" if always_uv else "off"
+            data["Always Require UV"] = "On" if always_uv else "Off"
 
         remaining_creds = ctap2.info.remaining_disc_creds
         if remaining_creds is not None:
-            info["Credential storage remaining"] = remaining_creds
+            data["Credential storage remaining"] = remaining_creds
 
         ep = ctap2.info.options.get("ep")
         if ep is not None:
-            info["Enterprise Attestation"] = "enabled" if ep else "disabled"
-
+            data["Enterprise Attestation"] = "Enabled" if ep else "Disabled"
     else:
-        info["PIN"] = "not supported"
+        data["PIN"] = "Not supported"
 
     click.echo("\n".join(pretty_print(lines)))
 
@@ -254,7 +258,7 @@ def reset(ctx, force):
                 fips_reset(conn)
             else:
                 Ctap2(conn).reset()
-        logger.info("FIDO application data reset")
+        click.echo("FIDO application data reset.")
     except CtapError as e:
         if e.code == CtapError.ERR.ACTION_TIMEOUT:
             raise CliFail(
@@ -266,7 +270,7 @@ def reset(ctx, force):
                 "YubiKey is inserted."
             )
         else:
-            raise CliFail(f"Reset failed: {e.code.name}")
+            raise CliFail(f"Reset failed: {e.code.name}.")
     except ApduError as e:  # From fips_reset
         if e.code == SW.COMMAND_NOT_ALLOWED:
             raise CliFail(
@@ -322,7 +326,8 @@ def change_pin(ctx, pin, new_pin, u2f):
     6 characters long.
     """
 
-    is_fips = is_yk4_fips(ctx.obj["info"])
+    info = ctx.obj["info"]
+    is_fips = is_yk4_fips(info)
 
     if is_fips and not u2f:
         raise CliFail(
@@ -345,10 +350,19 @@ def change_pin(ctx, pin, new_pin, u2f):
             raise CliFail("PIN is not supported on this YubiKey.")
         client_pin = ClientPin(ctap2)
         min_len = ctap2.info.min_pin_length
+    if (
+        info._is_bio
+        and CAPABILITY.PIV in info.config.enabled_capabilities[TRANSPORT.USB]
+    ):
+        max_len = 8
+    else:
+        max_len = 63
 
     def _fail_if_not_valid_pin(pin=None, name="PIN"):
         if not pin or len(pin) < min_len:
-            raise CliFail(f"{name} must be at least {min_len} characters long")
+            raise CliFail(f"{name} must be at least {min_len} characters long.")
+        if len(pin) > max_len:
+            raise CliFail(f"{name} must be at most {max_len} characters long.")
 
     def prompt_new_pin():
         return click_prompt(
@@ -378,7 +392,7 @@ def change_pin(ctx, pin, new_pin, u2f):
             if e.code == CtapError.ERR.PIN_POLICY_VIOLATION:
                 raise CliFail("New PIN doesn't meet complexity requirements.")
             else:
-                _fail_pin_error(ctx, e, "Failed to change PIN: %s")
+                _fail_pin_error(ctx, e, "Failed to change PIN: %s.")
 
         except ApduError as e:
             if e.code == SW.VERIFY_FAIL_NO_RETRY:
@@ -386,17 +400,16 @@ def change_pin(ctx, pin, new_pin, u2f):
             elif e.code == SW.AUTH_METHOD_BLOCKED:
                 raise CliFail("PIN is blocked.")
             else:
-                raise CliFail(f"Failed to change PIN: SW={e.code:04x}")
+                raise CliFail(f"Failed to change PIN: SW={e.code:04x}.")
 
     def set_pin(new_pin):
-        _fail_if_not_valid_pin(new_pin)
         try:
             client_pin.set_pin(new_pin)
         except CtapError as e:
             if e.code == CtapError.ERR.PIN_POLICY_VIOLATION:
                 raise CliFail("New PIN doesn't meet complexity requirements.")
             else:
-                raise CliFail(f"Failed to set PIN: {e.code}")
+                raise CliFail(f"Failed to set PIN: {e.code}.")
 
     if not is_fips:
         if ctap2.info.options.get("clientPin"):
@@ -417,7 +430,7 @@ def change_pin(ctx, pin, new_pin, u2f):
             change_pin(pin, new_pin)
         else:
             set_pin(new_pin)
-    logger.info("FIDO PIN updated")
+    click.echo("FIDO PIN updated.")
 
 
 def _require_pin(ctx, pin, feature="This feature"):
@@ -454,7 +467,7 @@ def verify(ctx, pin):
                 pin, ClientPin.PERMISSION.GET_ASSERTION, "ykman.example.com"
             )
         except CtapError as e:
-            raise CliFail(f"PIN verification failed: {e}")
+            raise CliFail(f"PIN verification failed: {e}.")
     elif is_yk4_fips(ctx.obj["info"]):
         try:
             fips_verify_pin(ctx.obj["conn"], pin)
@@ -466,7 +479,7 @@ def verify(ctx, pin):
             elif e.code == SW.COMMAND_NOT_ALLOWED:
                 raise CliFail("PIN is not set.")
             else:
-                raise CliFail(f"PIN verification failed: {e.code.name}")
+                raise CliFail(f"PIN verification failed: {e.code.name}.")
     else:
         raise CliFail("This YubiKey does not support a FIDO PIN.")
     click.echo("PIN verified.")
@@ -488,7 +501,7 @@ def _init_config(ctx, pin):
                 pin, ClientPin.PERMISSION.AUTHENTICATOR_CFG
             )
         except CtapError as e:
-            _fail_pin_error(ctx, e, "PIN error: %s")
+            _fail_pin_error(ctx, e, "PIN error: %s.")
 
     return Config(ctap2, protocol, token)
 
@@ -500,21 +513,22 @@ def force_pin_change(ctx, pin):
     """
     Force the PIN to be changed to a new value before use.
     """
-    options = ctx.obj.get("ctap2").info.options
-    if not options.get("setMinPINLength"):
+    options = ctx.obj["ctap2"].info.options if "ctap2" in ctx.obj else None
+    if options is None or not options.get("setMinPINLength"):
         raise CliFail("Force change PIN is not supported on this YubiKey.")
     if not options.get("clientPin"):
         raise CliFail("No PIN is set.")
 
     config = _init_config(ctx, pin)
     config.set_min_pin_length(force_change_pin=True)
+    click.echo("Force PIN change set.")
 
 
 @access.command("set-min-length")
 @click.pass_context
 @click.option("-P", "--pin", help="PIN code")
 @click.option("-R", "--rp-id", multiple=True, help="RP ID to allow")
-@click.argument("length", type=click.IntRange(4))
+@click.argument("length", type=click.IntRange(4, 63))
 def set_min_pin_length(ctx, pin, rp_id, length):
     """
     Set the minimum length allowed for PIN.
@@ -522,8 +536,8 @@ def set_min_pin_length(ctx, pin, rp_id, length):
     Optionally use the --rp option to specify which RPs are allowed to request this
     information.
     """
-    info = ctx.obj["ctap2"].info
-    if not info.options.get("setMinPINLength"):
+    info = ctx.obj["ctap2"].info if "ctap2" in ctx.obj else None
+    if info is None or not info.options.get("setMinPINLength"):
         raise CliFail("Set minimum PIN length is not supported on this YubiKey.")
     if info.options.get("alwaysUv") and not info.options.get("clientPin"):
         raise CliFail(
@@ -534,6 +548,14 @@ def set_min_pin_length(ctx, pin, rp_id, length):
     if length < min_len:
         raise CliFail(f"Cannot set a minimum length that is shorter than {min_len}.")
 
+    dev_info = ctx.obj["info"]
+    if (
+        dev_info._is_bio
+        and CAPABILITY.PIV in dev_info.config.enabled_capabilities[TRANSPORT.USB]
+        and length > 8
+    ):
+        raise CliFail("Cannot set a minimum length that is longer than 8.")
+
     config = _init_config(ctx, pin)
     if rp_id:
         ctap2 = ctx.obj.get("ctap2")
@@ -544,6 +566,7 @@ def set_min_pin_length(ctx, pin, rp_id, length):
             )
 
     config.set_min_pin_length(min_pin_length=length, rp_ids=rp_id)
+    click.echo("Minimum PIN length set.")
 
 
 def _prompt_current_pin(prompt="Enter your current PIN"):
@@ -614,7 +637,7 @@ def _init_credman(ctx, pin):
     try:
         token = client_pin.get_pin_token(pin, ClientPin.PERMISSION.CREDENTIAL_MGMT)
     except CtapError as e:
-        _fail_pin_error(ctx, e, "PIN error: %s")
+        _fail_pin_error(ctx, e, "PIN error: %s.")
 
     return CredentialManagement(ctap2, client_pin.protocol, token)
 
@@ -696,7 +719,7 @@ def creds_delete(ctx, credential_id, pin, force):
         ):
             try:
                 credman.delete_cred(cred_id)
-                logger.info("Credential deleted")
+                click.echo("Credential deleted.")
             except CtapError:
                 raise CliFail("Failed to delete credential.")
     else:
@@ -739,7 +762,7 @@ def _init_bio(ctx, pin):
     try:
         token = client_pin.get_pin_token(pin, ClientPin.PERMISSION.BIO_ENROLL)
     except CtapError as e:
-        _fail_pin_error(ctx, e, "PIN error: %s")
+        _fail_pin_error(ctx, e, "PIN error: %s.")
 
     return FPBioEnrollment(ctap2, client_pin.protocol, token)
 
@@ -798,7 +821,7 @@ def bio_enroll(ctx, name, pin):
                 )
             elif e.code == CtapError.ERR.USER_ACTION_TIMEOUT:
                 raise CliFail("Failed to add fingerprint due to user inactivity.")
-            raise CliFail(f"Failed to add fingerprint: {e.code.name}")
+            raise CliFail(f"Failed to add fingerprint: {e.code.name}.")
     logger.info("Fingerprint template registered")
     click.echo("Capture complete.")
     bio.set_name(template_id, name)
@@ -829,7 +852,7 @@ def bio_rename(ctx, template_id, name, pin):
         raise CliFail(f"No fingerprint matching ID={template_id}.")
 
     bio.set_name(key, name)
-    logger.info("Fingerprint template renamed")
+    click.echo("Fingerprint template renamed.")
 
 
 @bio.command("delete")
@@ -856,7 +879,7 @@ def bio_delete(ctx, template_id, pin, force):
         # Match using template_id as NAME
         matches = [k for k in enrollments if enrollments[k] == template_id]
         if len(matches) == 0:
-            raise CliFail(f"No fingerprint matching ID={template_id}")
+            raise CliFail(f"No fingerprint matching ID={template_id}.")
         elif len(matches) > 1:
             raise CliFail(
                 f"Multiple matches for NAME={template_id}. "
@@ -868,9 +891,9 @@ def bio_delete(ctx, template_id, pin, force):
     if force or click.confirm(f"Delete fingerprint {_format_fp(key, name)}?"):
         try:
             bio.remove_enrollment(key)
-            logger.info("Fingerprint template deleted")
+            click.echo("Fingerprint template deleted.")
         except CtapError as e:
-            raise CliFail(f"Failed to delete fingerprint: {e.code.name}")
+            raise CliFail(f"Failed to delete fingerprint: {e.code.name}.")
 
 
 @fido.group("config")
@@ -887,12 +910,19 @@ def toggle_always_uv(ctx, pin):
     """
     Toggles the state of Always Require User Verification.
     """
-    options = ctx.obj.get("ctap2").info.options
-    if "alwaysUv" not in options:
+    options = ctx.obj.get("ctap2").info.options if "ctap2" in ctx.obj else None
+    if not options or "alwaysUv" not in options:
         raise CliFail("Always Require UV is not supported on this YubiKey.")
 
+    info = ctx.obj["info"]
+    if CAPABILITY.FIDO2 in info.fips_capable:
+        raise CliFail("Always Require UV can not be disabled on this YubiKey.")
+
+    always_uv = options["alwaysUv"]
+
     config = _init_config(ctx, pin)
     config.toggle_always_uv()
+    click.echo(f"Always Require UV is {'off' if always_uv else 'on'}.")
 
 
 @config.command("enable-ep-attestation")
@@ -902,8 +932,8 @@ def enable_ep_attestation(ctx, pin):
     """
     Enables Enterprise Attestation for Authenticators pre-configured to support it.
     """
-    options = ctx.obj.get("ctap2").info.options
-    if "ep" not in options:
+    options = ctx.obj.get("ctap2").info.options if "ctap2" in ctx.obj else None
+    if not options or "ep" not in options:
         raise CliFail("Enterprise Attestation is not supported on this YubiKey.")
     if options.get("alwaysUv") and not options.get("clientPin"):
         raise CliFail(
@@ -913,3 +943,4 @@ def enable_ep_attestation(ctx, pin):
 
     config = _init_config(ctx, pin)
     config.enable_enterprise_attestation()
+    click.echo("Enterprise Attestation enabled.")
