@@ -25,8 +25,17 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-import click
-import logging
+from yubikit.core import TRANSPORT
+from yubikit.core.smartcard import ApduError, SW, SmartCardConnection
+from yubikit.oath import (
+    OathSession,
+    CredentialData,
+    OATH_TYPE,
+    HASH_ALGORITHM,
+    parse_b32_key,
+    _format_cred_id,
+)
+from yubikit.management import CAPABILITY
 from .util import (
     CliFail,
     click_force_option,
@@ -39,19 +48,15 @@ from .util import (
     prompt_timeout,
     EnumChoice,
     is_yk4_fips,
-)
-from yubikit.core.smartcard import ApduError, SW, SmartCardConnection
-from yubikit.oath import (
-    OathSession,
-    CredentialData,
-    OATH_TYPE,
-    HASH_ALGORITHM,
-    parse_b32_key,
-    _format_cred_id,
+    pretty_print,
+    get_scp_params,
 )
 from ..oath import is_steam, calculate_steam, is_hidden, delete_broken_credential
 from ..settings import AppData
 
+from typing import Dict, List, Any
+import click
+import logging
 
 logger = logging.getLogger(__name__)
 
@@ -76,14 +81,21 @@ def oath(ctx):
 
     \b
       Set a password for the OATH application:
-      $ ykman oath access change-password
+      $ ykman oath access change
     """
 
     dev = ctx.obj["device"]
     conn = dev.open_connection(SmartCardConnection)
     ctx.call_on_close(conn.close)
-    ctx.obj["session"] = OathSession(conn)
+
+    scp_params = get_scp_params(ctx, CAPABILITY.OATH, conn)
+
+    ctx.obj["session"] = OathSession(conn, scp_params)
     ctx.obj["oath_keys"] = AppData("oath_keys")
+    info = ctx.obj["info"]
+    is_fips = CAPABILITY.OATH in info.fips_capable
+    ctx.obj["fips_unready"] = is_fips and CAPABILITY.OATH not in info.fips_approved
+    ctx.obj["no_scp"] = is_fips and dev.transport == TRANSPORT.NFC and not scp_params
 
 
 @oath.command()
@@ -93,16 +105,22 @@ def info(ctx):
     Display general status of the OATH application.
     """
     session = ctx.obj["session"]
-    version = session.version
-    click.echo(f"OATH version: {version[0]}.{version[1]}.{version[2]}")
-    click.echo("Password protection: " + ("enabled" if session.locked else "disabled"))
+    info = ctx.obj["info"]
+    data: Dict[str, Any] = {"OATH version": "%d.%d.%d" % session.version}
+    lines: List[Any] = [data]
 
+    if CAPABILITY.OATH in info.fips_capable:
+        # This is a bit ugly as it makes assumptions about the structure of data
+        data["FIPS approved"] = CAPABILITY.OATH in info.fips_approved
+    elif is_yk4_fips(info):
+        data["FIPS approved"] = session.locked
+
+    data["Password protection"] = "enabled" if session.locked else "disabled"
     keys = ctx.obj["oath_keys"]
     if session.locked and session.device_id in keys:
-        click.echo("The password for this YubiKey is remembered by ykman.")
+        lines.append("The password for this YubiKey is remembered by ykman.")
 
-    if is_yk4_fips(ctx.obj["info"]):
-        click.echo(f"FIPS Approved Mode: {'Yes' if session.locked else 'No'}")
+    click.echo("\n".join(pretty_print(lines)))
 
 
 @oath.command()
@@ -134,7 +152,7 @@ def reset(ctx, force):
         keys.write()
         logger.info("Deleted remembered access key")
 
-    click.echo("Success! All OATH accounts have been deleted from the YubiKey.")
+    click.echo("Reset complete. All OATH accounts have been deleted from the YubiKey.")
 
 
 click_password_option = click.option(
@@ -203,6 +221,12 @@ def _init_session(ctx, password, remember, prompt="Enter the password"):
         raise CliFail("Password provided, but no password is set.")
 
 
+def _fail_scp(ctx, e):
+    if ctx.obj["no_scp"] and e.sw == SW.CONDITIONS_NOT_SATISFIED:
+        raise CliFail("Unable to manage OATH over NFC without SCP")
+    raise e
+
+
 @oath.group()
 def access():
     """Manage password protection for OATH."""
@@ -226,8 +250,13 @@ def change(ctx, password, clear, new_password, remember):
     Allows you to set or change a password that will be required to access the OATH
     accounts stored on the YubiKey.
     """
-    if clear and new_password:
-        ctx.fail("--clear cannot be combined with --new-password.")
+    if clear:
+        if new_password:
+            raise CliFail("--clear cannot be combined with --new-password.")
+
+        info = ctx.obj["info"]
+        if CAPABILITY.OATH in info.fips_capable:
+            raise CliFail("Removing the password is not allowed on YubiKey FIPS.")
 
     _init_session(ctx, password, False, prompt="Enter the current password")
 
@@ -263,8 +292,11 @@ def change(ctx, password, clear, new_password, remember):
         elif device_id in keys:
             del keys[device_id]
             keys.write()
-        session.set_key(key)
-        click.echo("Password updated.")
+        try:
+            session.set_key(key)
+            click.echo("Password updated.")
+        except ApduError as e:
+            _fail_scp(ctx, e)
 
 
 @access.command()
@@ -297,6 +329,7 @@ def remember(ctx, password):
         key = session.derive_key(password)
         try:
             _validate(ctx, key, True)
+            click.echo("Password remembered.")
         except Exception:
             raise CliFail("Authentication to the YubiKey failed. Wrong password?")
 
@@ -356,9 +389,7 @@ def _string_id(credential):
 
 
 def _error_multiple_hits(ctx, hits):
-    click.echo(
-        "Error: Multiple matches, please make the query more specific.", err=True
-    )
+    click.echo("Error: Multiple matches, make the query more specific.", err=True)
     click.echo("", err=True)
     for cred in hits:
         click.echo(_string_id(cred), err=True)
@@ -455,6 +486,11 @@ def add(
     SECRET  base32-encoded secret/key value provided by the server
     """
 
+    if ctx.obj["fips_unready"]:
+        raise CliFail(
+            "YubiKey FIPS must be in FIPS approved mode prior to adding accounts"
+        )
+
     digits = int(digits)
 
     if not secret:
@@ -482,7 +518,7 @@ def add(
 def click_parse_uri(ctx, param, val):
     try:
         return CredentialData.parse_uri(val)
-    except ValueError:
+    except (ValueError, KeyError):
         raise click.BadParameter("URI seems to have the wrong format.")
 
 
@@ -500,6 +536,11 @@ def uri(ctx, data, touch, force, password, remember):
     Use a URI to add a new account to the YubiKey.
     """
 
+    if ctx.obj["fips_unready"]:
+        raise CliFail(
+            "YubiKey FIPS must be in FIPS approved mode prior to adding accounts"
+        )
+
     if not data:
         while True:
             uri = click_prompt("Enter an OATH URI (otpauth://)")
@@ -523,16 +564,16 @@ def _add_cred(ctx, data, touch, force):
     version = session.version
 
     if not (0 < len(data.name) <= 64):
-        ctx.fail("Name must be between 1 and 64 bytes.")
+        raise CliFail("Name must be between 1 and 64 bytes.")
 
     if len(data.secret) < 2:
-        ctx.fail("Secret must be at least 2 bytes.")
+        raise CliFail("Secret must be at least 2 bytes.")
 
     if touch and version < (4, 2, 6):
         raise CliFail("Require touch is not supported on this YubiKey.")
 
     if data.counter and data.oath_type != OATH_TYPE.HOTP:
-        ctx.fail("Counter only supported for HOTP accounts.")
+        raise CliFail("Counter only supported for HOTP accounts.")
 
     if data.hash_algorithm == HASH_ALGORITHM.SHA512 and (
         version < (4, 3, 1) or is_yk4_fips(ctx.obj["info"])
@@ -560,14 +601,14 @@ def _add_cred(ctx, data, touch, force):
 
     try:
         session.put_credential(data, touch)
+        click.echo("OATH account added.")
     except ApduError as e:
         if e.sw == SW.NO_SPACE:
             raise CliFail("No space left on the YubiKey for OATH accounts.")
         elif e.sw == SW.COMMAND_ABORTED:
             # Some NEOs do not use the NO_SPACE error.
             raise CliFail("The command failed. Is there enough space on the YubiKey?")
-        else:
-            raise
+        _fail_scp(ctx, e)
 
 
 @accounts.command()
@@ -618,7 +659,7 @@ def code(ctx, show_hidden, query, single, password, remember):
 
     Generate codes from OATH accounts stored on the YubiKey.
     Provide a query string to match one or more specific accounts.
-    Accounts of type HOTP, or those that require touch, requre a single match to be
+    Accounts of type HOTP, or those that require touch, require a single match to be
     triggered.
     """
 
