@@ -41,10 +41,11 @@ from .core.otp import (
     OtpProtocol,
     CommandRejectedError,
 )
-from .core.smartcard import AID, SmartCardConnection, SmartCardProtocol
+from .core.smartcard import AID, SmartCardConnection, SmartCardProtocol, ScpKeyParams
 
 import abc
 import struct
+import warnings
 from hashlib import sha1
 from threading import Event
 from enum import unique, IntEnum, IntFlag
@@ -129,7 +130,7 @@ class CFGFLAG(IntFlag):
     # Yubikey 2 and above
     SHORT_TICKET = 0x02  # Send truncated ticket (half length)
     STRONG_PW1 = 0x10  # Strong password policy flag #1 (mixed case)
-    STRONG_PW2 = 0x40  # Strong password policy flag #2 (subtitute 0..7 to digits)
+    STRONG_PW2 = 0x40  # Strong password policy flag #2 (substitute 0..7 to digits)
     MAN_UPDATE = 0x80  # Allow manual (local) update of static OTP
 
     # Yubikey 2.1 and above
@@ -633,8 +634,6 @@ class ConfigState:
 
 
 class _Backend(abc.ABC):
-    version: Version
-
     @abc.abstractmethod
     def close(self) -> None:
         ...
@@ -673,6 +672,7 @@ class _YubiOtpOtpBackend(_Backend):
 
 
 INS_CONFIG = 0x01
+INS_YK2_STATUS = 0x03
 
 
 class _YubiOtpSmartCardBackend(_Backend):
@@ -686,15 +686,18 @@ class _YubiOtpSmartCardBackend(_Backend):
 
     def write_update(self, slot, data):
         status = self.protocol.send_apdu(0, INS_CONFIG, slot, 0, data)
+        if not status:  # Some commands don't return status on some YubiKeys
+            status = self.protocol.send_apdu(0, INS_YK2_STATUS, 0, 0)
+
         prev_prog_seq, self._prog_seq = self._prog_seq, status[3]
         if self._prog_seq == prev_prog_seq + 1:
             return status
         if self._prog_seq == 0 and prev_prog_seq > 0:
             version = Version.from_bytes(status[:3])
-            if (4, 0) <= version < (5, 5):  # Programming state does not update
-                return status
             if status[4] & 0x1F == 0:
                 return status
+            if (5, 0, 0) <= version < (5, 4, 3):  # Programming state does not update
+                return status
         raise CommandRejectedError("Not updated")
 
     def send_and_receive(self, slot, data, expected_len, event=None, on_keepalive=None):
@@ -707,8 +710,14 @@ class _YubiOtpSmartCardBackend(_Backend):
 class YubiOtpSession:
     """A session with the YubiOTP application."""
 
-    def __init__(self, connection: Union[OtpConnection, SmartCardConnection]):
+    def __init__(
+        self,
+        connection: Union[OtpConnection, SmartCardConnection],
+        scp_key_params: Optional[ScpKeyParams] = None,
+    ):
         if isinstance(connection, OtpConnection):
+            if scp_key_params:
+                raise ValueError("SCP can only be used with SmartCardConnection")
             otp_protocol = OtpProtocol(connection)
             self._status = otp_protocol.read_status()
             self._version = otp_protocol.version
@@ -732,7 +741,9 @@ class YubiOtpSession:
                 self._version = max(mgmt_version, otp_version)
             else:
                 self._version = mgmt_version or otp_version
-            card_protocol.enable_touch_workaround(self._version)
+            card_protocol.configure(self._version)
+            if scp_key_params:
+                card_protocol.init_scp(scp_key_params)
             self.backend = _YubiOtpSmartCardBackend(
                 card_protocol, self._version, self._status[3]
             )
@@ -745,10 +756,20 @@ class YubiOtpSession:
         )
 
     def close(self) -> None:
+        """Close the underlying connection.
+
+        :deprecated: call .close() on the underlying connection instead.
+        """
+        warnings.warn(
+            "Deprecated: call .close() on the underlying connection instead.",
+            DeprecationWarning,
+        )
         self.backend.close()
 
     @property
     def version(self) -> Version:
+        """The version of the Yubico OTP application,
+        typically the same as the YubiKey firmware."""
         return self._version
 
     def get_serial(self) -> int:
